/***********************************************************************

  c35sms: Serial.h
  by Martin Decky
  
  Redistribution of this file is permitted under the GNU Public License.

  1.0		Initial version

***********************************************************************/


#ifndef __SERIAL_H__
#define __SERIAL_H__


#include <string>
#include <list>
#include <cc++/serial.h>
#include <cc++/slog.h>


#ifdef CCXX_NAMESPACES
using namespace ost;
#endif
using namespace std;


/** SMS number representation
 *
 * Check whether a number is a valid
 * GSM number and store it.
 *
 */
class SMSNumber {
	public:
		SMSNumber();                /**< Default constructor, always throws invalid_argument */
		SMSNumber(string & number); /**< Conversion constructor */
		
		SMSNumber & operator=(string & number); /**< Conversion assign operator */
		operator const string & () const;       /**< Conversion operator */
		
		char numType() const; /**< Get GSM type of the number */
		
	private:
		string value; /**< Value of number */
		
		void assign(string & number); /**< Assign new value, check for correctness */
};


typedef list<SMSNumber> SMSNumbers; /**< List of SMS numbers */
typedef list<string> Commands;      /**< Commands list */


/** Serial communication stream
 *
 */
class SMSSerial : public TTYStream {
	public:
		SMSSerial(const string & device, const timeout_t timeout, const unsigned long baud, Slog * logger = NULL); /**< Constructor */
		
		SMSNumbers::size_type sendSMS(SMSNumbers & numbers, string & msg); /**< Send SMS message */
		void resetMobile();                                                /**< Reset mobile phone */
	private:
		Slog * syslog; /**< Syslog facility */
		Commands cmds; /**< Commands previously sent to the serial line */
		
		string sendCommand(const bool ack, const char * cmd, const string & params = ""); /**< Send AT command and wait for OK response */
		string getResponse(const bool ack);                                               /**< Get response */
		string getPrompt();                                                               /**< Get prompt */
};


#endif
