/***********************************************************************

  c35sms: Server.h
  by Martin Decky
  
  Redistribution of this file is permitted under the GNU Public License.

  1.0		Initial version

***********************************************************************/


#include "Server.h"


/** Constructor
 *
 * Create a TCP socket for accepting connections
 * from the client.
 *
 * Note successful initialization of the daemon
 * to syslog facility, noting binded address and port.
 *
 * @param bind   Inet address to bind.
 * @param port   Port to bind.
 * @param logger Syslog facility to use, NULL for no logging.
 *
 */
SMSServer::SMSServer(const InetAddress & bind, tpport_t port, const Addresses & allowed, Slog * logger)
	: TCPSocket(bind, port) {
	
	syslog = logger;
	allow = allowed;
	
	/* Log server startup */
	if (syslog) {
		tpport_t port;
		InetHostAddress addr = getLocal(&port);
		
		(*syslog)(Slog::levelInfo) << "Listening at " << addr << ", port " << port << endl;
	}
}


/** Connection filter
 *
 * Check whether accept the connection
 * from the client (according its address).
 *
 * @param ia   Inet address of the client.
 * @param port Port of the client.
 *
 * @return True if the connection can be accepted.
 *
 */
bool SMSServer::onAccept(const InetHostAddress & ia, tpport_t port) {
	/* Check whether host's IP address
	   is the list of allowed hosts */
	bool ret = false;
	for (Addresses::iterator it = allow.begin(); it != allow.end(); it++)
		if (ia == *it)
			ret = true;
	
	/* Log filter result */
	if (syslog)
		(*syslog)(Slog::levelInfo) << "Connection from " << ia << ":" << port << " " << (ret ? "accepted" : "rejected" ) << endl;
		
	return ret;
}

